==> bqpackage-1.0.999.tar.gz: bqpackage-1.0.999/bqpackage/selenium_base/selenium_actions.py <==
"""
This module wrapping basic selenium actions
"""
import re

import allure
from time import sleep

from selenium.webdriver.common.by import By
from selenium.webdriver import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver import Chrome

MAX_PAGES = 50
MAX_CLICKS = 30
MAX_NOT_EXISTS = 2
GLOBAL_MAX = 1000
CLICK_INTERVAL = 3


@allure.step
class SeleniumActions:

    @allure.step('Selenium actions initialization')
    def __init__(self, driver):
        self._list_of_elements = []
        self._web_element_type = "webelement"
        self._temp_element = None
        self._wait_seconds = 0
        self.driver: Chrome = driver
        self.global_timeout = 240

    def set_global_timeout(self, global_timeout: int):
        self.global_timeout = global_timeout

    @allure.step
    def pagination(self, container: str, next_btn: str, disabled: str):
        """
        Loop pages until reaches the end

        :arg container : Container
        :arg next_btn : Next button element
        :arg disabled : next button disabled
        :returns True->successfully paginated, False->error
        """
        for i in self.pagination_iterator(container=container, next_btn=next_btn, disabled=disabled):
            pass
        return True

    @allure.step
    def pagination_iterator(self, container: str, next_btn: str, disabled: str):
        """
        Loop pages, yield every page for future consume

        :arg container : Container
        :arg next_btn : Next button element
        :arg disabled : next button disabled
        :returns page text->when there's text, False->error
        """
        i = 0
        container_text = self.get_text(element=container, strip=True)
        try:
            next_attrs = self.get_element(element=f'{container}{next_btn}').get_attribute('class')
            while next_attrs is not None and disabled not in next_attrs:
                yield container_text
                self.click(element=f'{container}{next_btn}')
                i += 1
                container_text = self.get_text(element=container, strip=True)
                next_attrs = self.get_element(element=f'{container}{next_btn}').get_attribute('class')
                if not i < MAX_PAGES:
                    raise RuntimeError(f'looping pages reached maximum {i}/{MAX_PAGES}')
        except AttributeError:
            pass

        # exhausted
        yield container_text

    @allure.step
    def text_in_pagination(self, container: str, next_btn: str, disabled: str, text: str):
        """
        Loop pages until reaches the end/criteria met (text in page)

        :arg container : Container
        :arg next_btn : Next button element
        :arg disabled : next button disabled
        :arg text : Text to look for
        :returns Text->Text found on current page, False->error
        """
        text = re.sub(r"\s+", "", text)
        for container_text in self.pagination_iterator(container=container, next_btn=next_btn, disabled=disabled):
            if text in container_text:
                return text

        return False

    @allure.step
    def elements_iterator(self, element: str):
        """
        Loop elements from the same kind, yield every element for future consume => (element)[i]

        For ex. if your element type is "//div[contains(text(), 'blabla')]" and there plenty of it
        You will pass this element and the iterator wil yield you a wrap of
        (//div[contains(text(), 'blabla')])[i] each time

        :arg element : kind of element
        :returns WebElement->if found, False->element not found
        """
        i = 1
        el = self.get_element(element=f'({element})[{i}]')
        while el:
            yield el
            i += 1
            if not i < GLOBAL_MAX:
                raise RuntimeError(f'looping elements reached maximum {i}/{GLOBAL_MAX}')
            el = self.get_element(element=f'({element})[{i}]')

        # exhausted
        return el

    @allure.step
    def wait_for_text(self, location=None, text=None):
        """
        wait for a text to be presented inside/without element (currently searching by xpath!)
        driver max waiting time is 30 sec.
        :arg location : element to search inside
        :arg text : text to look for
        :returns True->Text found on current element/non element, False->error
        """
        if location:
            WebDriverWait(self.driver, self.global_timeout).until(
                EC.text_to_be_present_in_element((By.XPATH, location), text))
        else:
            WebDriverWait(self.driver, self.global_timeout).until(EC.text_to_be_present_in_element
                                                                 (locator=(By.XPATH, "//body"), text_=text))
        return True

    @allure.step
    def where_is_element(self, element: str):
        """
        ping you when element have been found (only XPATH)
        :param element: XPATH
        :param time: longer than 240?
        :return: None if element have been found, Raise timeout if time passed
        """
        WebDriverWait(self.driver, self.global_timeout).until(EC.presence_of_element_located((By.XPATH, element)))
        return True

    @allure.step
    def get_element(self, element):
        """
        find element
        :param element: XPATH
        :return: Single element
        """
        self.where_is_element(element=element)
        return self.driver.find_element(By.XPATH, value=element)

    @allure.step
    def get_elements(self, element):
        """
        Get all the elements
        :param element: XPATH
        :return: List of elements
        """
        self.where_is_element(element=element)
        return self.driver.find_elements(By.XPATH, element)

    @allure.step
    def click(self, element, proof=None):
        """
        click on element
        :arg element : element to be clicked (XPATH)
        :arg proof : optional function that returns True when some requirement satisfied, for ex. where_is_element
        :returns True->Clicked False->error
        """
        element_to_click = self.get_element(element)
        if proof:
            self.vclick(element=element_to_click, proof=proof)
        else:
            element_to_click.click()
        return True

    @allure.step
    def vclick(self, element, proof: dict):
        """
        loop of clicking an element until a proof (text exist on page).
        :arg element : element to be clicked
        :arg proof : optional function that returns True when some requirement satisfied
        :returns True->Clicked and proof exist False->error
        """
        clicks = 0
        clicked = False
        while not clicked:
            try:
                element.click()
            except Exception as e:
                pass

            sleep(CLICK_INTERVAL)
            clicks += 1

            if clicks > MAX_CLICKS:
                raise RuntimeError(f'maximum clicks passed limit {clicks}/{MAX_CLICKS}')

            clicked = proof()

        if not clicked:
            raise RuntimeError(f"failed to click on element: {element}")

        allure.attach(str(clicks), 'Clicks', allure.attachment_type.TEXT)
        return True

    @allure.step
    def hover(self, element, wait=0):
        sleep(wait)
        self._temp_element = self.get_element(element)
        try:
            if self._temp_element is not None:
                hover = ActionChains(self.driver).move_to_element(self._temp_element)
                hover.perform()
                return True
        except Exception as e:
            raise RuntimeError(f'failed to hover {element}')

    @allure.step
    def send_keys(self, element=None, text=None):
        text = str(text)
        if not element:
            ActionChains(self.driver).send_keys(text).perform()
            return True
        self.where_is_element(element)
        element = self.get_element(element)
        element.send_keys(text)
        return True

    @allure.step
    def get_text(self, element, strip=False):
        self.where_is_element(element)
        element = self.get_element(element)
        allure.attach(element.text, 'Text', allure.attachment_type.TEXT)
        if strip:
            return re.sub(r"\s+", "", element.text)
        return element.text

    @allure.step
    def get_page_source(self):
        source = self.driver.page_source
        allure.attach(source, 'Page source', allure.attachment_type.TEXT)
        return source

==> bqpackage-1.0.9992.tar.gz: bqpackage-1.0.9992/bqpackage/selenium_base/selenium_actions.py <==
"""
This module wrapping basic selenium actions
"""
import re

import allure
from time import sleep

from selenium.webdriver.common.by import By
from selenium.webdriver import ActionChains
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver import Chrome

MAX_PAGES = 50
MAX_CLICKS = 30
MAX_NOT_EXISTS = 2
GLOBAL_MAX = 1000


@allure.step
class SeleniumActions:

    @allure.step('Selenium actions initialization')
    def __init__(self, driver):
        self._list_of_elements = []
        self._web_element_type = "webelement"
        self._temp_element = None
        self._wait_seconds = 0
        self.driver: Chrome = driver
        self.global_timeout = 240

    def set_global_timeout(self, global_timeout: int):
        self.global_timeout = global_timeout

    @allure.step
    def pagination(self, container: str, next_btn: str, disabled: str):
        """
        Loop pages until reaches the end

        :arg container : Container
        :arg next_btn : Next button element
        :arg disabled : next button disabled
        :returns True->successfully paginated, False->error
        """
        for i in self.pagination_iterator(container=container, next_btn=next_btn, disabled=disabled):
            pass
        return True

    @allure.step
    def pagination_iterator(self, container: str, next_btn: str, disabled: str):
        """
        Loop pages, yield every page for future consume

        :arg container : Container
        :arg next_btn : Next button element
        :arg disabled : next button disabled
        :returns page text->when there's text, False->error
        """
        i = 0
        container_text = self.get_text(element=container, strip=True)
        try:
            next_attrs = self.get_element(element=f'{container}{next_btn}').get_attribute('class')
            while next_attrs is not None and disabled not in next_attrs:
                yield container_text
                self.click(element=f'{container}{next_btn}')
                i += 1
                container_text = self.get_text(element=container, strip=True)
                next_attrs = self.get_element(element=f'{container}{next_btn}').get_attribute('class')
                if not i < MAX_PAGES:
                    raise RuntimeError(f'looping pages reached maximum {i}/{MAX_PAGES}')
        except AttributeError:
            pass

        # exhausted
        yield container_text

    @allure.step
    def text_in_pagination(self, container: str, next_btn: str, disabled: str, text: str):
        """
        Loop pages until reaches the end/criteria met (text in page)

        :arg container : Container
        :arg next_btn : Next button element
        :arg disabled : next button disabled
        :arg text : Text to look for
        :returns Text->Text found on current page, False->error
        """
        text = re.sub(r"\s+", "", text)
        for container_text in self.pagination_iterator(container=container, next_btn=next_btn, disabled=disabled):
            if text in container_text:
                return text

        return False

    @allure.step
    def elements_iterator(self, element: str):
        """
        Loop elements from the same kind, yield every element for future consume => (element)[i]

        For ex. if your element type is "//div[contains(text(), 'blabla')]" and there plenty of it
        You will pass this element and the iterator wil yield you a wrap of
        (//div[contains(text(), 'blabla')])[i] each time

        :arg element : kind of element
        :returns WebElement->if found, False->element not found
        """
        i = 1
        el = self.get_element(element=f'({element})[{i}]')
        while el:
            yield el
            i += 1
            if not i < GLOBAL_MAX:
                raise RuntimeError(f'looping elements reached maximum {i}/{GLOBAL_MAX}')
            el = self.get_element(element=f'({element})[{i}]')

        # exhausted
        return el

    @allure.step
    def wait_for_text(self, location=None, text=None):
        """
        wait for a text to be presented inside/without element (currently searching by xpath!)
        driver max waiting time is 30 sec.
        :arg location : element to search inside
        :arg text : text to look for
        :returns True->Text found on current element/non element, False->error
        """
        if location:
            WebDriverWait(self.driver, self.global_timeout).until(
                EC.text_to_be_present_in_element((By.XPATH, location), text))
        else:
            WebDriverWait(self.driver, self.global_timeout).until(EC.text_to_be_present_in_element
                                                                 (locator=(By.XPATH, "//body"), text_=text))
        return True

    @allure.step
    def where_is_element(self, element: str):
        """
        ping you when element have been found (only XPATH)
        :param element: XPATH
        :param time: longer than 240?
        :return: None if element have been found, Raise timeout if time passed
        """
        WebDriverWait(self.driver, self.global_timeout).until(EC.presence_of_element_located((By.XPATH, element)))
        return True

    @allure.step
    def get_element(self, element):
        """
        find element
        :param element: XPATH
        :return: Single element
        """
        self.where_is_element(element=element)
        return self.driver.find_element(By.XPATH, value=element)

    @allure.step
    def get_elements(self, element):
        """
        Get all the elements
        :param element: XPATH
        :return: List of elements
        """
        self.where_is_element(element=element)
        return self.driver.find_elements(By.XPATH, element)

    @allure.step
    def click(self, element, proof=None, tries=MAX_CLICKS, maximum=None):
        """
        click on element
        :arg element : element to be clicked (XPATH)
        :arg proof : optional function that returns True when some requirement satisfied, for ex. where_is_element
        :arg tries : optional arg, how many tries do you want before throwing exception ?
        :arg maximum : optional arg, maximum waiting time before throwing exception
        :returns True->Clicked False->error
        """
        element_to_click = self.get_element(element)
        if proof:
            self.vclick(element=element_to_click, proof=proof, tries=tries, maximum=maximum)
        else:
            element_to_click.click()
        return True

    @allure.step
    def vclick(self, element, proof, tries, maximum):
        """
        loop of clicking an element until a proof (text exist on page).
        :arg element : element to be clicked
        :arg proof : optional function that returns True when some requirement satisfied
        :arg tries : optional arg, how many tries do you want before throwing exception ?
        :arg maximum : optional arg, maximum waiting time before throwing exception
        :returns True->Clicked and proof exist False->error
        """
        if tries > MAX_CLICKS:
            raise RuntimeError(f'cannot assign tries {tries} > {MAX_CLICKS}')

        temp_global = self.global_timeout
        if maximum:
            self.global_timeout = maximum

        clicks = 0
        clicked = False
        while not clicked:
            try:
                element.click()
            except Exception as e:
                pass

            clicks += 1

            if clicks > tries:
                raise RuntimeError(f'maximum clicks passed limit {clicks}/{tries}')

            try:
                clicked = proof()
            except Exception as e:
                pass

        self.global_timeout = temp_global
        if not clicked:
            raise RuntimeError(f"failed to click on element: {element}")

        allure.attach(str(clicks), 'Clicks', allure.attachment_type.TEXT)
        return True

    @allure.step
    def hover(self, element, wait=0):
        sleep(wait)
        self._temp_element = self.get_element(element)
        try:
            if self._temp_element is not None:
                hover = ActionChains(self.driver).move_to_element(self._temp_element)
                hover.perform()
                return True
        except Exception as e:
            raise RuntimeError(f'failed to hover {element}')

    @allure.step
    def send_keys(self, element=None, text=None):
        text = str(text)
        if not element:
            ActionChains(self.driver).send_keys(text).perform()
            return True
        self.where_is_element(element)
        element = self.get_element(element)
        element.send_keys(text)
        return True

    @allure.step
    def get_text(self, element, strip=False):
        self.where_is_element(element)
        element = self.get_element(element)
        allure.attach(element.text, 'Text', allure.attachment_type.TEXT)
        if strip:
            return re.sub(r"\s+", "", element.text)
        return element.text

    @allure.step
    def get_page_source(self):
        source = self.driver.page_source
        allure.attach(source, 'Page source', allure.attachment_type.TEXT)
        return source
